%-------------------------------------------------------------------------%
% @Input:       f   -   vector with initial values
%               p   -   degree
%               z   -   point of evaluation
%               n   -   const. in three-term recurrence relation 
%               s   -   shift paramter in three-term recurrence relation
% @Output:      [f0 f1 ... fn], nu
% @Description: Calculates the solution of a three-term recurrence relation
%               via backward evaluation. Returns also maximal nu.
% @author:      Pascal Heiter
% @date:        2010/05/10
%-------------------------------------------------------------------------%

function [f,nu] = backward3term(f,p,z,n,s)

%Preperations
tol = 1e-14;
told = [realmax;1];  
%Set nu arbitrary (nu > p )
nu = p+2; 
nu_max = 5000;
%Set initial values
t = [1;0]; 

while nu < nu_max
  %Save only the last two entries  
  for i = nu:-1:p
    t= [((2*(i-s)+1)*z*t(1)-((i-s)-n+1)*t(2))/((i-s)+n); 
         t(1)];
    t = t./norm(t);
  end
  t = t./t(2);
  %Check, if t is a good approximation 
  if sum(abs(told-t)<tol*abs(t)) == 2
      break;
  end  
  told = t;
  nu = nu + 1;
end

j = length(f)-1;
tmp = f(end);
f(p:p+1) = t;
%Calculate the remaining values
for i=p-1:-1:j+1
  f(i)=((2*(i-s)+1)*z*f(i+1)-((i-s)-n+1)*f(i+2))/((i-s)+n);
end
%Multiply with tmp/f(j+1), because f(j+1) was overwritten
f(j+1:end) = f(j+1:end) * tmp/f(j+1);